"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createHybridModeProjectProviderFactory = void 0;
const simpleProject_1 = require("@volar/language-server/lib/project/simpleProject");
const simpleProjectProvider_1 = require("@volar/language-server/lib/project/simpleProjectProvider");
const language_core_1 = require("@vue/language-core");
const language_service_1 = require("@vue/language-service");
const utils_1 = require("@vue/typescript-plugin/lib/utils");
function createHybridModeProjectProviderFactory(sys) {
    return (context, servicePlugins, getLanguagePlugins) => {
        const serviceEnvs = new language_core_1.FileMap(sys.useCaseSensitiveFileNames);
        const tsconfigProjects = new language_core_1.FileMap(sys.useCaseSensitiveFileNames);
        const simpleProjects = new language_core_1.FileMap(sys.useCaseSensitiveFileNames);
        context.onDidChangeWatchedFiles(({ changes }) => {
            for (const change of changes) {
                if (tsconfigProjects.has(change.uri)) {
                    tsconfigProjects.get(change.uri)?.then(project => project.dispose());
                    tsconfigProjects.delete(change.uri);
                    context.reloadDiagnostics();
                }
            }
        });
        return {
            async getProject(uri) {
                const workspaceFolder = (0, simpleProjectProvider_1.getWorkspaceFolder)(uri, context.workspaceFolders);
                let serviceEnv = serviceEnvs.get(workspaceFolder);
                if (!serviceEnv) {
                    serviceEnv = (0, simpleProjectProvider_1.createServiceEnvironment)(context, workspaceFolder);
                    serviceEnvs.set(workspaceFolder, serviceEnv);
                }
                const fileName = serviceEnv.typescript.uriToFileName(uri);
                const projectInfo = (await (0, utils_1.searchNamedPipeServerForFile)(fileName))?.projectInfo;
                if (projectInfo?.kind === 1) {
                    const tsconfig = projectInfo.name;
                    const tsconfigUri = serviceEnv.typescript.fileNameToUri(tsconfig);
                    if (!tsconfigProjects.has(tsconfigUri)) {
                        tsconfigProjects.set(tsconfigUri, (async () => {
                            const languagePlugins = await getLanguagePlugins(serviceEnv, {
                                typescript: {
                                    configFileName: tsconfig,
                                    host: {
                                        getScriptFileNames() {
                                            return [];
                                        },
                                    },
                                    sys: {
                                        ...sys,
                                        version: 0,
                                        async sync() {
                                            return 0;
                                        },
                                        dispose() { },
                                    },
                                },
                            });
                            return createTSConfigProject(context, serviceEnv, languagePlugins, servicePlugins);
                        })());
                    }
                    return await tsconfigProjects.get(tsconfigUri);
                }
                else {
                    if (!simpleProjects.has(workspaceFolder)) {
                        simpleProjects.set(workspaceFolder, (() => {
                            return (0, simpleProject_1.createSimpleServerProject)(context, serviceEnv, servicePlugins, getLanguagePlugins);
                        })());
                    }
                    return await simpleProjects.get(workspaceFolder);
                }
            },
            async getProjects() {
                return Promise.all([
                    ...tsconfigProjects.values(),
                    ...simpleProjects.values(),
                ]);
            },
            async reloadProjects() {
                for (const project of [
                    ...tsconfigProjects.values(),
                    ...simpleProjects.values(),
                ]) {
                    (await project).dispose();
                }
                tsconfigProjects.clear();
            },
        };
        function createTSConfigProject(context, serviceEnv, languagePlugins, servicePlugins) {
            let languageService;
            return {
                getLanguageService,
                getLanguageServiceDontCreate: () => languageService,
                dispose() {
                    languageService?.dispose();
                },
            };
            function getLanguageService() {
                if (!languageService) {
                    const language = (0, language_core_1.createLanguage)(languagePlugins, false, uri => {
                        const script = context.documents.get(uri);
                        if (script) {
                            language.scripts.set(uri, script.languageId, script.getSnapshot());
                        }
                        else {
                            language.scripts.delete(uri);
                        }
                    });
                    languageService = (0, language_service_1.createLanguageService)(language, servicePlugins, serviceEnv);
                }
                return languageService;
            }
        }
    };
}
exports.createHybridModeProjectProviderFactory = createHybridModeProjectProviderFactory;
//# sourceMappingURL=hybridModeProject.js.map